# **********************************************************************
# Simulation of Aerial wildlife Survey using transect or roset sampling 
# Authors : Lejeune P. (p.lejeune@uliege.be 
#           Linchant J.(julie.linchant@gmail.com)
# Version : 2022/09/07
# Structure of the script :
# 1.1 Libraries : declare the libraries used in the script
# 1.2 Initilize memory : emptying the workspace
# 1.3 Funtions : place where functions are grouped
# 1.4 Input data, parameters  & output folder : define main path, aoi layer, ...
# 2.1 Build rosets with different orientation : generate roset layers with different orientation
# 2.2 Build transect layers with 1500 & 3000 m of intersdistance & 
# 3.1 Simulation of Antelope surveys
# 3.1.1. Simulation with transects
# 3.1.1. Simulation with rosets
# 3.2 Simulation of Buffalo surveys
# 3.2.1. Simulation with transects
# 3.2.2. Simulation with rosets
#
# Before running the differents parts of the script, you need to create
# a folder to store the project (see variable path0 in section 1.4)
# Then the differents steps have to be run one by one, with some check 
# between each step
#
# **********************************************************************

# 1.1 Libraries ------------------------
library(sf)
library(raster)  
library(future)
library(parallel)
library(dplyr)
library(gdalUtils) 
library(nngeo)
library(DescTools)

# 1.2 Initialize memory ----------------------------

rm(list=ls())


# 1.3 Functions ---------------------------------

antelope_group_generator <- function(m0,n0,N0){

  gr_size=rpois(n0, m0)
  
  # remove group with 0 animals
  gr_size=gr_size[gr_size>0]

  # store the list of group size in a df
  # the list is stopped when the size of population is reached
  # the last group can be truncated if necessary
  df=data.frame(size=integer())
  i=0
  while (sum(df$size)<N0){
    i=i+1
    size=gr_size[i]
    if((sum(df$size)+size)>N0){
      size=N0-sum(df$size)
    }
    k=nrow(df)+1
    df[k,1]=size
  }
  
  n_group=nrow(df)
  df$id=seq.int(n_group)
  
  return(df)
}

antelope_group_locator = function(i){
  require(sf)
  # generate the position of groups inside the aoi
  nb_group=nrow(group_of_antelope)
  rnd0=st_sample(aoi, size=nb_group, type = "random", exact = TRUE)  
  rnd0=st_as_sf(rnd0)
  rnd0$id=seq.int(nrow(rnd0))
  group=left_join(rnd0,group_of_antelope,by=c("id"="id"))
  names(group)[3]="geometry"
  st_geometry(group)="geometry"
  f_out=paste0(path_out,"/antelope_",i_date,"_",i,".gpkg")
  st_write(group,f_out,delete_layer=T,quiet=T)
  return()
}

buffalo_group_generator = function(N0,proba,n1,n2){
  
  df=data.frame(size=integer())
  i=0
  while (sum(df$size)<N0){
    i=i+1
    rnd=runif(1)
    cl=1
    for (j in 2:5){
      if(rnd>proba[j-1] & rnd<=proba[j]){
        cl=j
      }
    }
    s1=n1[cl]
    s2=n2[cl]
    size=as.integer(runif(1,s1,s2))
    if((sum(df$size)+size)>N0){
      size=N0-sum(df$size)
    }
    k=nrow(df)+1
    df[k,1]=size
  }
  return(df)
  
}


buffalo_group_locator = function(i){
  require (nngeo)
  require(sf)
  require(DescTools)

  # generate the position of groups inside the aoi
  list_ell=NULL
  
  for (j in 1:nrow(group_of_buffalo)){
    rnd0=st_sample(aoi, size=1, type = "random", exact = TRUE)    
    x0=st_coordinates(rnd0)[1]
    y0=st_coordinates(rnd0)[2]
    area=group_of_buffalo$size[j]*apa
    b=((area/2/pi)^0.5)
    a=2*b

    ell0=st_ellipse(rnd0,a,b,res=15)
    ell1=Rotate(st_coordinates(ell0),theta=runif(1, min=0, max=pi))
    df0=as.data.frame(ell1$x)
    names(df0)="x"
    df0$y=ell1$y
    ell1=st_as_sf(df0,coords=c("x","y"),crs=crs0)
    ell1=summarize(ell1)
    buff=st_buffer(ell1,dist=0.1)
    ell2=st_convex_hull(buff)
    #plot(ell2)
    ell2$size=group_of_buffalo$size[j]
    
    list_ell=dplyr::bind_rows(list_ell,ell2)
  }

  f_out=paste0(path0,"/output/buffalo_groups/buffalo_1_",i,".gpkg")
  st_write(list_ell,f_out,delete_layer=T,quiet=T)
  return()
}


antelope_simulate_transect_survey  = function(i){
  require(sf)
  require(dplyr)
  # load a set of observation
  i00 = as.integer(runif(1,1,nb_rep_location)) # initialize counter (between 1 - 10000)
  f_in=paste0(path_in,"/antelope_",i_date,"_",i00,".gpkg")
  group0=st_read(f_in,quiet=T)
  group0=dplyr::select(group0,id_group=id,size)
  # intersect groups with transect bands
  intersect=st_intersection(group0,sf_survey)
  intersect$xi=st_coordinates(intersect)[,1]
  intersect$yi=st_coordinates(intersect)[,2]
  
  # compute distance to border
  intersect$d2border=as.numeric(st_distance(intersect,sf_survey_border))
  names(intersect) 
  intersect$size[intersect$d2border<1]=round(intersect$size[intersect$d2border<3]/2,0)

  intersect=as.data.frame(st_drop_geometry(intersect))
  # agregate obs by transect
  tab1 = intersect %>% 
    group_by(id) %>% 
    summarize(y = sum(size))
  df=left_join(sf_survey,tab1,by=c("id" = "id"))
  df$y[is.na(df$y)]=0
  df=as.data.frame(st_drop_geometry(df))
  df=dplyr::select(df,id,a=surf,y)
  df$ya=df$y*df$a
  df$date=i_date
  df$dist=i_dist
  df$repet=i
  return(df)
}

antelope_simulate_roset_survey  = function(i){
  require(sf)
  require(dplyr)
  # repetition of flights
  df_all=NULL
  for(i_flight in 1:nb_max_flights){
    # randomly select a roset flight plan
    #azim=as.integer(runif(1,0,8))*10
    azim=sample(c(0,10,20,30,40,50,60,70,80),1)
    f_in=paste0(path0,"/output/rosets/r_pol_",azim , ".gpkg")
    sf_survey=st_read(f_in,quiet=T)
    sf_survey$id=sf_survey$site*10+sf_survey$petal
    sf_survey_border=st_cast(sf_survey,"MULTILINESTRING")
    sf_survey_border=summarize(sf_survey_border)
    # load a set of observation
    i0 = as.integer(runif(1,1,nb_rep_location)) # initialize counter (between 1 - 10000)
    f_in=paste0(path_in,"/antelope_",i_date,"_",i0,".gpkg")
    group0=st_read(f_in,quiet=T)
    group0=dplyr::select(group0,id_group=id,size)
    
    # intersect groups with rosette bands
    intersect=st_intersection(group0,sf_survey)
    intersect$xi=st_coordinates(intersect)[,1]
    intersect$yi=st_coordinates(intersect)[,2]
    # compute distance to border
    intersect$d2border=as.numeric(st_distance(intersect,sf_survey_border))
    names(intersect) 
    intersect$size[intersect$d2border<1]=round(intersect$size[intersect$d2border<3]/2,0)
    intersect=as.data.frame(st_drop_geometry(intersect))

    # agregate rosets feature at the petal level
    ros0 = sf_survey %>% 
            group_by(id) %>% 
            summarize()
    ros0$surf=as.numeric(st_area(ros0))/1000000
    ros0=as.data.frame(st_drop_geometry(ros0))
    intersect$d0=((intersect$x0-intersect$xi)^2+(intersect$y0-intersect$yi)^2)^0.5
    intersect$w0=(intersect$d0*2*pi/12/120)
    intersect$w0[intersect$border==1]=2
    intersect$size_w0=intersect$size*intersect$w0/weight_fact
      

    tab1 = intersect %>% 
      group_by(id) %>% 
      summarize(y = sum(size_w0))
    
    df=left_join(ros0,tab1,by=c("id" = "id"))
    df$y[is.na(df$y)]=0
    df=dplyr::select(df,id,a=surf,y)
    df$ya=df$y*df$a
    df$date=i_date
    df$flight=i_flight
    df$repet=i
    df$azim=azim
    df_all=bind_rows(df_all,df)
  }    
  return(df_all)

}

buffalo_simulate_transect_survey  = function(i){
  require(sf)
  require(dplyr)
  # load a set of observation
  i00 = as.integer(runif(1,1,nb_rep_location)) # initialize counter (between 1 - 10000)
  f_in=paste0(path_in,"/buffalo_1_",i00,".gpkg")
  group0=st_read(f_in,quiet=T)
  group0$id=seq.int(nrow(group0))
  group0=dplyr::select(group0,id_group=id,size)
  group0$surf_group=as.numeric(st_area(group0))

  # intersect groups with transect bands and compute the number of animal inside strip
  intersect=st_intersection(group0,sf_survey)
  
  intersect$surf_int=as.numeric(st_area(intersect))
  intersect$size_in=round(intersect$size/intersect$surf_group*intersect$surf_int,0)
  #intersect$size_in=(intersect$size/intersect$surf_group*intersect$surf_int)
  intersect=as.data.frame(st_drop_geometry(intersect))
  # convert polygons to point 
  #centro=st_point_on_surface(intersect)
  
  
  #centro$xi=st_coordinates(centro)[,1]
  #centro$yi=st_coordinates(centro)[,2]
  #intersect=as.data.frame(st_drop_geometry(centro))
  # compute distance to center
  #names(intersect)

  # agregate obs by transect
  tab1 = intersect %>% 
    group_by(id) %>% 
    summarize(y = sum(size_in))
  df=left_join(sf_survey,tab1,by=c("id" = "id"))
  df$y[is.na(df$y)]=0
  df=dplyr::select(df,id,a=surf,y)
  df$ya=df$y*df$a
  df$dist=i_dist
  df$repet=i
  df=as.data.frame(st_drop_geometry(df))
  return(df)
}

buffalo_simulate_roset_survey  = function(i){
  require(sf)
  require(dplyr)
  # repetition of flights
  df_all=NULL
  for(i_flight in 1:nb_max_flights){
    # randomly select a roset flight plan
    azim=as.integer(runif(1,0,8))*10
    f_in=paste0(path0,"/output/rosets/r_pol_",azim , ".gpkg")
    sf_survey=st_read(f_in,quiet=T)
    sf_survey$id=sf_survey$site*10+sf_survey$petal
    # load a set of observation
    i0 = as.integer(runif(1,1,nb_rep_location)) # initialize counter (between 1 - 10000)
    f_in=paste0(path_in,"/buffalo_1_",i0,".gpkg")
    group0=st_read(f_in,quiet=T)
    group0$id=seq.int(nrow(group0))
    group0=dplyr::select(group0,id_group=id,size)
    group0$surf_group=as.numeric(st_area(group0))
    
    # intersect groups with rosette bands
    intersect=st_intersection(group0,sf_survey)
    intersect$surf_int=as.numeric(st_area(intersect))
    intersect$size_in=(intersect$size/intersect$surf_group*intersect$surf_int)
    centro=st_point_on_surface(intersect)
    # coordinates of the group center (or at least the part of the group inside the roset )
    intersect$xi=st_coordinates(centro)[,1]
    intersect$yi=st_coordinates(centro)[,2]
    intersect=as.data.frame(st_drop_geometry(intersect))

    # agregate rosets feature at the petal level
    ros0 = sf_survey %>% 
      group_by(id) %>% 
      summarize()
    ros0$surf=as.numeric(st_area(ros0))/1000000
    ros0=as.data.frame(st_drop_geometry(ros0))
    intersect$d0=((intersect$x0-intersect$xi)^2+(intersect$y0-intersect$yi)^2)^0.5
    intersect$w0=(intersect$d0*2*pi/12/120)
    intersect$w0[intersect$border==1]=2
    intersect$size_w0=intersect$size_in*intersect$w0/weight_fact
    
    
    tab1 = intersect %>% 
      group_by(id) %>% 
      summarize(y = sum(size_w0))
    
    df=left_join(ros0,tab1,by=c("id" = "id"))
    df$y[is.na(df$y)]=0
    df=dplyr::select(df,id,a=surf,y)
    df$ya=df$y*df$a
    df$flight=i_flight
    df$repet=i
    df$azim=azim
    df_all=bind_rows(df_all,df)
  }    
  return(df_all)
}


# 1.4 Input data, parameters  & output folder -------------
# main path
path0="d:/survey_simulation" # main folder of the project
# layer with the limits of the study area (stored in a sub-folder "data_in")
f_aoi = paste0(path0,"/data_in/garamba_area.gpkg")
aoi=st_read(f_aoi)
aoi_sqkm=as.numeric(st_area(aoi))/1000000
crs0=32635 # EPSG code of the study area
# create output folder if necessary
path_out=paste0(path0,"/output")
if(!dir.exists(path_out)){
  dir.create(path_out)
  path_ant=paste0(path_out,"/antelope_groups")
  dir.create(path_ant)
  path_buf=paste0(path_out,"/buffalo_groups")
  dir.create(path_buf)
}



# 2.1 Build rosets with different orientation -------

x_center=c(764970,780455) # coordinates of the 2 roset centers
y_center=c(424790,431030)
dist0=8 # size of petals (in km)

# holes around the rostes centers
center=as.data.frame(cbind(x_center,y_center))
center=st_as_sf(center, coords = c("x_center", "y_center"), crs = crs0)
hole=st_buffer(center,dist=250)
hole=summarize(hole)

# loop to generate rosets with different orientations (from 0 to 80 by 10)
d0=dist0*1000 # petal length in m
for (azim  in seq(0,80,10)){
  print(azim)
  for (site in 1:2){
    x0=x_center[site]
    y0=y_center[site]
    
    for (petal in 1:6){
      az1=90-(petal-1)*60-azim
      az2=az1-30
      x1=x0+d0*cos(az1/180*pi)
      y1=y0+d0*sin(az1/180*pi)
      x2=x0+d0*cos(az2/180*pi)
      y2=y0+d0*sin(az2/180*pi)
      # 3 edges for 1 petal, edge n3 is the border
      line1 = rbind(c(x0,y0), c(x1,y1))
      line2 = rbind(c(x0,y0), c(x2,y2))
      line3 = rbind(c(x1,y1), c(x2,y2))
      
      line1=st_linestring(line1)
      line1=st_sfc(line1)
      line2=st_linestring(line2)
      line2=st_sfc(line2)
      line3=st_linestring(line3)
      line3=st_sfc(line3)
      
      line=rbind(line1,line2)
      line=rbind(line,line3)
      
      sf = st_sf(st_sfc(line,crs=crs0))
      names(sf)=c("geometry")
      st_geometry(sf)="geometry"
      sf$border=c(0,0,1) # edge3=border
      sf$petal=petal
      sf$site=site
      if(site==1 & petal==1){
        roset=sf
      }else{
        roset=rbind(roset,sf)
      }
    }
  }
  
  # centerline
  f1=paste0(path0,"/output/rosets/r_line_",azim , ".gpkg")
  st_write(roset,f1,delete_layer = T,quiet=T)
  
  # strip  
  buff0=st_buffer(roset,dist=60)
  roset=st_difference(buff0,hole)
  roset=st_intersection(roset,aoi)
  roset=dplyr::select(roset,site,petal,border)
  # remove the overlaping part between border and other parts of the roset"
  border=roset[roset$border==1,]
  remove_border=summarize(border)
  roset=st_difference(roset,remove_border)
  roset=dplyr::select(roset,site,petal,border)
  roset=rbind(roset,border)
  # connect each petal to its center
  center$x0=st_coordinates(center)[,1]
  center$y0=st_coordinates(center)[,2]
  center
  roset=st_join(roset, center, join = st_nearest_feature)
  f2=paste0(path0,"/output/rosets/r_pol_",azim , ".gpkg")
  st_write(roset,f2,delete_layer = T,quiet=T)
  
  
}

# 2.2 Build transects with 1500 & 3000 m of intersdistance -----------
list_dist_trans=c(1500,3000)
df_trans=data.frame(dist=numeric(),surf=numeric())

# coordinates to drive the transects creation
y1=405000 # min y
y2=447000 # max y
x1=740000 # min x

for (d0 in list_dist_trans) {
  print(d0)
  f_line=paste0(path0, "/output/transects/trans_line_",d0,".gpkg")
  f_pol=paste0(path0, "/output/transects/trans_pol_",d0,".gpkg")
  trans=NULL 
  i=0
  x0=x1
  while(x0<800000){
    i=i+1
    sfg = st_linestring(rbind(c(x0, y1), c(x0, y2)))
    sfc=st_sfc(sfg,crs=crs0)
    df <- data.frame(id = i)
    sf=st_sf(cbind(sfc, df))
    trans=rbind(trans,sf)
    x0=x0+d0
  }
  
  
  
  # convert center lines to strips (left and write)
  buff500=st_buffer(trans,dist=250)
  buff200=st_buffer(trans,dist=100)
  buff500=summarize(buff500)
  buff200=summarize(buff200)

  strip=st_difference(buff500,buff200)
  head(strip)
  strip=st_cast(strip,"POLYGON")
  
  
  strip=st_intersection(strip,aoi)
  strip=dplyr::select(strip,geometry)
  strip$surf=as.numeric(st_area(strip))/1000000
  strip$id=seq.int(nrow(strip))
  
  # save the results in gpkg files
  st_write(trans,f_line,delete_layer = T,quiet=T)
  st_write(strip,f_pol,delete_layer = T,quiet=T)

  
}







# 2.3 Generate 100000 repetitions for date1 & date2 of antelope population groups location ------

fN=3183 # Antelope population size
mean_size=c(1.84,4.71) # Antelope population mean group size for each date
nb_rep=100000 # number of repetition

path_out=paste0(path0,"/output/antelope_groups")
for (i_date in 1:2){
  print(i_date)
  group_of_antelope=antelope_group_generator(mean_size[i_date],2000,N)
  
  plan(multisession)
  res0 = 1:nb_rep %>%
     furrr::future_map(.f = antelope_group_locator, .progress = T) %>%
     bind_rows()
}
  
  
  
  
  for (i_repet in 1:nbrepet){
    print(i_repet)
    group=antelope_group_locator(group_of_antelope,aoi)
    names(group)[3]="geometry"
    st_geometry(group)="geometry"
    f_out=paste0("d:/julie_simul/repet_antelope_2021/Ant_",i_date,"_",i_repet,".gpkg")
    st_write(group,f_out,delete_layer=T)
  }
# 2.4 Generate 100000 repetitions for buffalo population ------
N=4817 # size of buffalo population
proba=c(0.701,0.808,0.868,0.940,0.991) # realtive frequence for each size class
n1=c(1,26,51,101,201,301)  # lower classes limit
n2=c(25,50,100,200,300,350) # upper classes limit
apa=118 # area per animal (in sqm)
nb_rep=100000 # number of repetition
path_out=paste0(path0,"/output/buffalo_groups")

# create the groups of animals
group_of_buffalo=buffalo_group_generator(N,proba,n1,n2)


# generate repetition of groups location inside the aoi
plan(multisession)
res0 = 1:nb_rep %>%
  furrr::future_map(.f = buffalo_group_locator, .progress = T) %>%
  bind_rows()





# ----------------------
# 3.1 Simulation of Antelope surveys ------------------------------------
# population parameters
N=3183
(true_density=N/aoi_sqkm)
weight_fact=14.3 # mean weight factor for roset sampling
path_in=paste0(path0,"/output/antelope_groups")
list_dist=c(1500,3000)
nb_rep_simulation=10000 # number of reptition for the simulations
nb_rep_location=100000 # number of repetition for the group locations
nb_max_flights=4 # number of flight repetitions (for the rosets only)

path_in=paste0(path0,"/output/antelope_groups")




# 3.1.1 Antelope Transects surveys ---------------

plan(multisession)
df_all=NULL
for (i_date in 1:2){
  print(i_date)
  for(i_dist in 1:2){
    dist0=list_dist[i_dist]
    # read the transects layer
    f_trans=paste0(path0, "/output/transects/trans_pol_",dist0,".gpkg")
    sf_survey=st_read(f_trans,quiet=T)
    surf_survey=sum(sf_survey$surf)
    
    res0 = 1:nb_rep_simulation %>%
      furrr::future_map(.f = antelope_simulate_transect_survey, .progress = T) %>%
      bind_rows()
    df_all=bind_rows(df_all,res0)
  }
}

summary_table_tr=NULL # df to store the results
for(i_date in 1:2){
  print(i_date)
  for(i_dist in 1:2){
    print(i_dist)
    result=NULL
    for(i_repet in 1:nb_rep_simulation){
      df=df_all[df_all$repet==i_repet & df_all$dist==i_dist & df_all$date==i_date,]
      Dest=sum(df$y)/sum(df$a)
      rate=sum(df$a)/aoi_sqkm
      suma=sum(df$a)
      sumy=sum(df$y)
      a=mean(df$a)
      s2y=var(df$y)
      s2a=var(df$a)
      sya=cov(df$y,df$a)
      n=nrow(df)
      VarD=(1-rate)/n/a^2*(s2y+Dest^2*s2a-2*Dest*sya)
      fact_nN=(1-rate)/n/a^2
      ectD=(VarD)^0.5
      err=Dest-true_density
      cv_Jolly=ectD/Dest*100
      res0=data.frame(i_date,i_dist,i_repet,rate,Dest,ectD,sumy,suma,err,cv_Jolly)
      result=bind_rows(result,res0)
    }
    
    rate = mean(result$rate)*100
    mean_d= round(mean(result$Dest),3)
    std_d = round(sd(result$Dest),3)
    cv_d = round(sd(result$Dest)/mean(result$Dest)*100,1)
    bias  = round(mean(result$err)/true_density*100,2)
    cv_Jolly=mean(result$cv_Jolly)
    test=t.test(result$err,mu=0)
    pvalue=test$p.value
    line_result=data.frame(i_date,i_dist,mean_d,std_d,cv_d,bias,pvalue,cv_Jolly)
    summary_table_tr=rbind(summary_table_tr,line_result)
    
  }
}

summary_table_tr
f_out=paste0(path0, "/output/result_antelope_transect.csv")
write.table(summary_table_tr,f_out,col.names = T,row.names = F,sep=";")


# 3.1.2 Antelope Rosets surveys ---------------

# the process is parallelized a the repetition level
# the output df_all dataframe has 1 line per petal and per repetition
# for each flight, a roset with a random orientation is selected
# for each run, the located groups are then overlaied with the rosets strips
df_all=NULL
plan(multisession)
nb_rep_simulation=10000
for (i_date in 1:2){
  print(i_date)
  res0 = 1:nb_rep_simulation %>%
    furrr::future_map(.f = antelope_simulate_roset_survey, .progress = T) %>%
    bind_rows()
  df_all=bind_rows(df_all,res0)
}


# after the overlay process, data are used to compute estimated density & associated errors

summary_table_r=NULL # df to store the results
for(i_date in 1:2){
  print(i_date)
  for(nb_flight in 1:nb_max_flights){
    print(nb_flight)
    result=NULL
    for(i_repet in 1:nb_rep_simulation){
      df=df_all[df_all$repet==i_repet & df_all$flight<=nb_flight & df_all$date==i_date,]
      Dest=sum(df$y)/sum(df$a)
      rate=round(sum(df$a)/aoi_sqkm,3)
      suma=sum(df$a)
      sumy=sum(df$y)
      a=mean(df$a)
      s2y=var(df$y)
      s2a=var(df$a)
      sya=cov(df$y,df$a)
      n=nrow(df)
      VarD=(1-rate)/n/a^2*(s2y+Dest^2*s2a-2*Dest*sya)
      err=Dest-true_density
      if(sum(df$y)==0){
        ectD=0
        cv_Jolly=0
      }else{
        ectD=(VarD)^0.5
        cv_Jolly=ectD/Dest*100
      }
      azim=df$azim[1]
      res0=data.frame(i_date,nb_flight,i_repet,rate,Dest,ectD,sumy,suma,err,cv_Jolly,n,azim)
      result=bind_rows(result,res0)
    }
    
    t1= result %>% group_by(azim) %>% summarize(cvJ=mean(cv_Jolly),nb=n())
    t1
    
    rate = mean(result$rate)*100
    mean_d= round(mean(result$Dest),3)
    std_d = round(sd(result$Dest),3)
    cv_d = round(sd(result$Dest)/mean(result$Dest)*100,2)
    bias  = round(mean(result$err)/true_density*100,2)
    cv_Jolly=mean(result$cv_Jolly)
    test=t.test(result$err,mu=0)
    pvalue=test$p.value
    line_result=data.frame(i_date,nb_flight,rate,mean_d,std_d,cv_d,bias,pvalue,cv_Jolly)
    summary_table_r=rbind(summary_table_r,line_result)

  }
}

summary_table_r

f_out=paste0(path0, "/output/result_antelope_roset.csv")
write.table(summary_table_r,f_out,col.names = T,row.names = F,sep=";")

summary(result$Dest)



# 3.2 Simulation of Buffalo surveys --------------------
N=4817 # size of buffalo population
(true_density=N/aoi_sqkm)
apa=118 # area per animal (in sqm)
weight_fact=14.3 # mean weight factor for roset sampling
path_in=paste0(path0,"/output/buffalo_groups")
list_dist=c(1500,3000)
nb_rep_simulation=10000
nb_rep_location=100000
nb_max_flights=4

# 3.2.1 Buffalo Transect surveys ---------------------------

plan(multisession)
df_all=NULL
nb_rep_simulation=10000
for(i_dist in 1:2){
  dist0=list_dist[i_dist]
  # read the transects layer
  f_trans=paste0(path0, "/output/transects/trans_pol_",dist0,".gpkg")
  sf_survey=st_read(f_trans,quiet=T)
  surf_survey=sum(sf_survey$surf)
  res0 = 1:nb_rep_simulation %>%
    furrr::future_map(.f = buffalo_simulate_transect_survey, .progress = T) %>%
    bind_rows()
  df_all=bind_rows(df_all,res0)
}

summary_table_tr=NULL # df to store the results
for(i_dist in 1:2){
  print(i_dist)
  result=NULL
  for(i_repet in 1:nb_rep_simulation){
      df=df_all[df_all$repet==i_repet & df_all$dist==i_dist  ,]
      Dest=sum(df$y)/sum(df$a)
      rate=sum(df$a)/aoi_sqkm
      suma=sum(df$a)
      sumy=sum(df$y)
      a=mean(df$a)
      s2y=var(df$y)
      s2a=var(df$a)
      sya=cov(df$y,df$a)
      n=nrow(df)
      VarD=(1-rate)/n/a^2*(s2y+Dest^2*s2a-2*Dest*sya)
      ectD=(VarD)^0.5
      err=Dest-true_density
      cv_Jolly=ectD/Dest*100
      res0=data.frame(i_repet,rate,Dest,ectD,sumy,suma,err,cv_Jolly)
      result=bind_rows(result,res0)
    }
      
  rate = mean(result$rate)*100
  mean_d= round(mean(result$Dest),3)
  std_d = round(sd(result$Dest),3)
  cv_d = round(sd(result$Dest)/mean(result$Dest)*100,1)
  bias  = round(mean(result$err)/true_density*100,2)
  cv_Jolly=mean(result$cv_Jolly)
  test=t.test(result$err,mu=0)
  pvalue=test$p.value
  line_result=data.frame(i_dist,rate,mean_d,std_d,cv_d,bias,pvalue,cv_Jolly)
  summary_table_tr=rbind(summary_table_tr,line_result)
}

summary_table_tr

f_out=paste0(path0, "/output/result_buffalo_transect.csv")
write.table(summary_table_tr,f_out,col.names = T,row.names = F,sep=";")



# 3.2.2 Buffalo Roset surveys ---------------------------

plan(multisession)
df_all=NULL
res0 = 1:nb_rep_simulation %>%
  furrr::future_map(.f = buffalo_simulate_roset_survey, .progress = T) %>%
  bind_rows()
df_all=bind_rows(df_all,res0)
nrow(df_all)

summary_table_r=NULL # df to store the results
for(nb_flight in 1:nb_max_flights){
  print(nb_flight)
  result=NULL
  for(i_repet in 1:nb_rep_simulation){
    df=df_all[df_all$repet==i_repet & df_all$flight<=nb_flight ,]
    Dest=sum(df$y)/sum(df$a)
    rate=sum(df$a)/aoi_sqkm
    suma=sum(df$a)
    sumy=sum(df$y)
    a=mean(df$a)
    s2y=var(df$y)
    s2a=var(df$a)
    sya=cov(df$y,df$a)
    n=nrow(df)
    VarD=(1-rate)/n/a^2*(s2y+Dest^2*s2a-2*Dest*sya)
    err=Dest-true_density
    if(sum(df$y)==0){
      ectD=0
      cv_Jolly=0
    }else{
      ectD=(VarD)^0.5
      cv_Jolly=ectD/Dest*100
    }
    res0=data.frame(nb_flight,i_repet,rate,Dest,ectD,sumy,suma,err,cv_Jolly,n)
    result=bind_rows(result,res0)
  }
  rate = mean(result$rate)*100
  mean_d= round(mean(result$Dest),3)
  std_d = round(sd(result$Dest),3)
  cv_d = round(sd(result$Dest)/mean(result$Dest)*100,1)
  bias  = round(mean(result$err)/true_density*100,2)
  cv_Jolly=mean(result$cv_Jolly)
  test=t.test(result$err,mu=0)
  pvalue=test$p.value
  nbrep=nrow(result)
  line_result=data.frame(nb_flight,mean_d,std_d,cv_d,bias,pvalue,cv_Jolly,nbrep)
  summary_table_r=rbind(summary_table_r,line_result)
}

summary_table_r

f_out=paste0(path0, "/output/result_buffalo_roset.csv")
write.table(summary_table_r,f_out,col.names = T,row.names = F,sep=";")


